"""base entities extension for skillmat

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"
_ = unicode

from datetime import date, timedelta

from cubicweb.entities import authobjs


SKILLS = {0 : _("I don't want to learn this technology"),
          1 : _('I know nothing about this technology'),
          2 : _('Just beginning whith this technology'),
          3 : _('I start on this technology, and I can help'),
          4 : _('I am doing well, selfworking'),
          5 : _('I can conduct a training course')}


class SkillmatCWUserMixIn(object):

    def skill_info(self, skillname):
        rset = self._cw.execute('Any M,R WHERE M is Masters, M rate R, M foruser U, '
                                'M skill S, S name %(name)s, U eid %(u)s',
                                {'name': skillname, 'u': self.eid})
        if rset:
            return rset.get_entity(0, 0)
        return None

    def add_masters_url(self, skilleid, **kwargs):
        linkto = ('foruser:%s:subject' % self.eid,
                  'skill:%s:subject' % skilleid)
        return self._cw.build_url('add/Masters', __linkto=linkto, **kwargs)

    def mastered_skills(self, threshold=5):
        """return the list of mastered skills
        """
        rql = 'Any S,N WHERE M skill S, S name N, M rate >= %(r)s, M foruser U, U eid %(u)s'
        rset = self._cw.execute(rql, {'r': threshold, 'u': self.eid})
        return rset.entities()

class SkillmatCWUser(SkillmatCWUserMixIn, authobjs.CWUser):
    pass



"""template-specific forms/views/actions/components"""
from cubicweb.web import uicfg
from cubicweb.web.formfields import IntField

from cubes.skillmat.entities.base import SKILLS

RATEVOCAB = [(l, unicode(v)) for v, l in sorted(SKILLS.items())]
uicfg.autoform_field_kwargs.tag_attribute(
    ('Masters', 'rate'),
    {'choices': RATEVOCAB,
     'internationalizable': True})

uicfg.autoform_section.tag_subject_of(('Talk', 'attended_by', '*'), 'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('Talk', 'attended_by', '*'), 'muledit', 'attributes')



"""eskilmat specific entities

:organization: Logilab
:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from logilab.common.decorators import cached

from cubicweb.mixins import TreeMixIn
from cubicweb.interfaces import ICalendarable
from cubicweb.entities import AnyEntity

from cubes.folder.entities import Folder
from cubes.skillmat.interfaces import ISkill

class Masters(AnyEntity):
    __regid__ = 'Masters'
    fetch_attrs = ('rate', 'foruser', 'skill')

    def dc_title(self):
        return u'%s: %s' % (self.skill[0].name, self.rate)

    def dc_long_title(self):
        return u'%s / %s' % (self.foruser[0].dc_title(), self.dc_title())


class Technology(TreeMixIn, AnyEntity):
    __regid__ = 'Technology'
    __implements__ = (ISkill,)
    fetch_attrs = ('name',)
    tree_attribute = 'filed_under'

    def skill_categories(self):
        # XXX what if we have a skill tree with depth > 1
        return self.filed_under


class Skill(Folder):
    __implements__ = (ISkill,)

    def skill_categories(self):
        return [self]

class Talk(AnyEntity):
    __regid__ = 'Talk'
    __implements__ = (ICalendarable,)
    fetch_attrs = ('subject', 'description', 'talktime')

    def dc_title(self):
        return u'%s' % (self.subject)

    @property
    def start(self):
        return self.talktime

    @property
    def stop(self):
        return self.talktime



from yams.buildobjs import (EntityType, SubjectRelation, ObjectRelation,
                            String, Datetime, Int)

from yams.constraints import IntervalBoundConstraint

_ = unicode

from cubicweb.schemas.base import CWUser
from cubes.folder.schema import Folder

from cubicweb.schema import ERQLExpression

class Masters(EntityType): # XXX find a suitable name
    """relation promoted to EntityType to store the `rate` attribute
    """
    __permissions__ = {'read': ('managers', 'users'),
                       'add': ('managers', ERQLExpression('X foruser U')),
                       'update': ('managers', 'owners'),
                       'delete': ('managers',),
                       }
    rate = Int(required=True, constraints=[IntervalBoundConstraint(0, 5)])
    foruser = SubjectRelation('CWUser', cardinality='1*')
    skill = SubjectRelation(('Technology', 'Folder'), cardinality='1*')


class Technology(Folder):
    pass

# extend CWUser
CWUser.add_relation(SubjectRelation(('Technology', 'Folder')), name='wishes_to_learn')
CWUser.add_relation(SubjectRelation('Talk', cardinality='**'), name='interested_in')

class Talk(EntityType):
    """relation promoted to EntityType to store the `subject' attribute
    """
    subject        = String(required=True, fulltextindexed=True, indexed=True, maxsize=32)
    description    = String(fulltextindexed=True,
                            description=_('more detailed subject description'))
    talktime       = Datetime(description=_('estimated presentation date'))
    presented_by   = SubjectRelation('CWUser', cardinality='?*')
    attended_by    = SubjectRelation('CWUser', cardinality='**')
    talks_about    = SubjectRelation(('Technology','Folder'), cardinality='+*')
    comments       = ObjectRelation('Comment', cardinality='1*', composite='object')



"""skillmat boxes

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.selectors import implements, match_user_groups

from cubicweb.web.box import EditRelationBoxTemplate


class EditWishesBox(EditRelationBoxTemplate):
    __regid__ = 'edit-wishes'
    __select__ = (EditRelationBoxTemplate.__select__
                  & implements('CWUser') & match_user_groups('owners'))

    rtype = 'wishes_to_learn'
    target = 'object'

    def unrelated_entities(self, euser):
        """filter skills for which the user already knows"""
        entities = super(EditWishesBox, self).unrelated_entities(euser)
        mastered = set(entity.eid for entity in euser.mastered_skills())
        return [entity for entity in entities if entity.eid not in mastered]


class EditAttendedByBox(EditRelationBoxTemplate):
    __regid__ = 'edit-attended-by'
    __select__ = EditRelationBoxTemplate.__select__ & implements('Talk')

    rtype = 'attended_by'
    target = 'object'

    ## Todo : replace + et - in this box by a comprehensive label

  #   def w_related(self, box, entity):
#         """appends existing relations to the `box`"""
#         rql = 'DELETE S %s O WHERE S eid %%(s)s, O eid %%(o)s' % self.rtype
#         related = self.related_entities(entity)
#         for etarget in related:
#             box.append(self.box_item(entity, etarget, rql, u'assiste'))
#         return len(related)
    
#     def w_unrelated(self, box, entity):
#         """appends unrelated entities to the `box`"""
#         rql = 'SET S %s O WHERE S eid %%(s)s, O eid %%(o)s' % self.rtype
#         for etarget in self.unrelated_entities(entity):
#             box.append(self.box_item(entity, etarget, rql, u'n\'assite pas'))
  
   



"""skillmat interfaces"""

from logilab.common.interface import Interface

class ISkill(Interface):
    """base interface for Technology / Skill"""
    
    def skill_categories(self):
        """returns skill's categories"""




# pylint: disable-msg=W0622
"""cubicweb-skillmat application packaging information"""

modname = 'skillmat'
distname = 'cubicweb-skillmat'

numversion = (0, 7, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
copyright = '''Copyright (c) 2008-2010 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

author = 'Logilab'
author_email = 'contact@logilab.fr'

web = 'http://www.cubicweb.org/project/%s' % distname

short_desc = 'skill matrix component for the CubicWeb framework'
long_desc = '''This CubicWeb component models the skills of a team
(who can do what at which level of expertise).

CubicWeb is a semantic web application framework, see http://www.cubicweb.org
'''

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

__depends_cubes__ = {'folder': None,
                     'comment': None}
__depends__ = {'cubicweb': '>= 3.6.0'}
for key, value in __depends_cubes__.items():
    __depends__['cubicweb-'+key] = value
__use__ = tuple(__depends_cubes__)

# packaging

from glob import glob
from os import listdir as _listdir
from os.path import join, isdir

pyversions = ['2.4']

#from cubicweb.devtools.pkginfo import get_distutils_datafiles
CUBES_DIR = join('share', 'cubicweb', 'cubes')
THIS_CUBE_DIR = join(CUBES_DIR, 'skillmat')

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')]

try:
    data_files = [
        # common files
        [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],

        [join(THIS_CUBE_DIR, 'entities'), listdir('entities')],

        # client (web) files
        [join(THIS_CUBE_DIR, 'data'),  listdir('data')],
        [join(THIS_CUBE_DIR, 'i18n'),  listdir('i18n')],
        [join(THIS_CUBE_DIR, 'views'), listdir('views')],
        [join(THIS_CUBE_DIR, 'wdoc'), listdir('wdoc')],
        # Note: here, you'll need to add views' subdirectories if you want
        # them to be included in the debian package

        # server files
        [join(THIS_CUBE_DIR, 'migration'), listdir('migration')],
        ]
except OSError, xx:
    # we are in an installed directory
    pass

cube_eid = None # <=== FIXME if you need direct bug-subscription
